with unchecked_conversion;
with ada.text_io; use ada.text_io;
procedure BRUCH_16 is
---------------------------------------------------------------------------
-- Stichworte: Interne Darstellung von Gleitpunktzahlen, Ungenauigkeit von
-- Gleitpunktoperationen, unchecked_conversion.
---------------------------------------------------------------------------
-- Diese Prozedur liest drei Zahlen in die Gleitpunkt-Variablen
-- G1, G2 und G3. Gibt G1, G2, das Produkt G1 * G2 und G3 zweimal aus:
-- erst "normal" als Gleitpunkt-Zahlen und dann
-- als Folge von je 64 Bits (Nullen und Einsen).
-- Wenn man z.B. die drei Zahlen 0.1, 0.1 und 0.01 eingibt, dann kann
-- man an der Bit-Darstellung sehen, dass die Multiplikation von
-- Gleitpunkt-Zahlen nicht exakt ist (in den letzten Bits gibt es Ab-
-- wichungen zwischen 0.1 * 0.1 und 0.01).
---------------------------------------------------------------------------
-- Dieses Programm funktioniert nur, wenn Gleitpunktzahlen vom Typ GLEIT
-- (siehe unten) als 64-Bit-Zahlen dargestellt werden, und zwar mit
-- 1 Bit Vorzeichen, 11 Bit Exponent und 52 Bit Mantisse.
---------------------------------------------------------------------------
   -- Der Gleitpunkt-Typ GLEIT ist ziemlich genau (15 Dezimalziffern) und
   -- hat einen ziemlich grossen Bereich:
   type     GLEIT    is digits 15 range -1.0e307 .. +1.0e307;
   package  GLEIT_EA is new float_io(GLEIT); use GLEIT_EA;

   -- Eine Bit-Tabelle ist eine Reihung von booleschen Werten.
   -- GLEIT'size ist die Anzahl der Bits, die ein Wert vom Typ GLEIT
   -- belegt.
   -- Das "pragma pack(BIT_TAB)" fordert den Ausfuehrer auf, Werte vom
   -- Typ BIT_TAB mit moeglichst wenig Bits darzustellen (d.h. hier:
   -- mit nur 64 Bits statt mit 64 Bytes = 512 Bits).
   type     BIT_TAB  is array(integer range 1..GLEIT'size) of boolean;
   pragma   pack(BIT_TAB);

   -- Die Funktion CONVERT macht aus einer Bitkette vom Typ GLEIT eine
   -- Bitkette vom Typ BIT_TAB (ohne die Bits der Bitkette zu veraendern):
   function CONVERT is new unchecked_conversion(GLEIT, BIT_TAB);

   -- Ein paar Variable:
   G1, G2, G3, GERG : GLEIT;
   B1, B2, B3, BERG : BIT_TAB;
   ------------------------------------------------------------------------
   procedure PUT(B: BIT_TAB) is
   -- Diese Prozedur PUT gibt einen Wert vom Typ BIT_TAB
   -- als eine Folge von Nullen und Einsen aus,
   -- (0 fuer false, 1 fuer true). Zwei Blanks werden ausgegeben
   -- um die drei Teile einer Gleitpunkt-Zahl voneinander zu trennen:
   -- das Vorzeichen (1 Bit), den Exponenten (11 Bit) und
   -- die Mantisse (52 Bits).
   --------------------------------------------------------
   begin
      for I in reverse B'range loop
         if B(I) then
            put("1");     -- 1 fuer true
         else
            put("0");     -- 0 fuer false
         end if;
         -- Nach dem Vorzeichen-Bit und nach den
         -- 11 Bits des Exponenten wird ein Blank ausgegeben:
         if I = 53 or I = 64 then
            put(" ");
         end if;
      end loop;
   end PUT;
   ------------------------------------------------------------------------
begin -- GLEITBIT
   loop
      put("BRUCH_16: Bitte 3 Bruchzahlen eingeben " &
          "(oder 0.0 zum Beenden): ------------");
      new_line;
      put("G1:      "); get(G1);
      exit when G1 = 0.0;
      put("G2:      "); get(G2);
      put("G3:      "); get(G3);
      new_line;

      -- G1, G2, das Produkt G1 * G2 und G3 "normal" ausgeben:
      put("G1:      "); put(G1);    new_line;
      put("G2:      "); put(G2);    new_line;
      GERG := G1 * G2;
      put("G1 * G2: "); put(GERG);  new_line;
      put("G3:      "); put(G3);    new_line;
      new_line;

      -- Die vier Gleitpunkt-Zahlen in Bit-Tabellen umwandeln:
      B1   := CONVERT(G1);
      B2   := CONVERT(G2);
      B3   := CONVERT(G3);
      BERG := CONVERT(GERG);

      -- Die vier Bit-Tabellen ausgeben:
      put("G1:      "); PUT(B1);    new_line;
      put("G2:      "); PUT(B2);    new_line;
      put("G1 * G2: "); PUT(BERG);  new_line;
      put("G3:      "); PUT(B3);    new_line;
      new_line;
   end loop;
   put_line("BRUCH_16: Das war's erstmal!");
end BRUCH_16;
