with ada.text_io, ada.sequential_io;
procedure DARST_05 is
---------------------------------------------------------------------------
-- Demo, wie man das Layout eines Verbundtyps mit Darstellungsklauseln
-- (representation clauses) festlegen kann. Ein Verbund des Typs
-- V_TYP belegt eine ganze Zahl von Speichereinheiten (storage units),
-- mindestens aber 43 Bit. Einige werden nicht benutzt. 
---------------------------------------------------------------------------
   type V_TYP is record              -- Ein Verbundtyp
      K1 : natural range 0..31;      --  5 Bit
      K2 : string(1..3);             -- 24 Bit
      K3 : integer range -30..+30;   --  6 Bit
   end record;
   -- Verbund-Darstellungsklausel (record representation clause),
   -- die Bits 0 bis 2 und die Bits 32 bis 36 bleiben unbenutzt:
   for V_TYP use record
      K1 at 0 range  3.. 7;          --  5 Bit
      K2 at 0 range  8..31;          -- 24 Bit
      K3 at 0 range 37..42;          --  6 Bit
   end record;
   -- Groessenattribut festlegen (attribut definition clause)
   -- for V_TYP'size use 64;
   ------------------------------------------------------------------------
   -- Pakete zum Ein-/Ausgeben von Daten:
   package INTEGER_EA is new ada.text_io.integer_io(num => integer);
   package   V_TYP_EA is new ada.sequential_io(element_type => V_TYP);
   ------------------------------------------------------------------------
   procedure PUT_VERBUND(ITEM: V_TYP) is
   -- Gibt ITEM in Textform zur aktuellen Ausgabe aus:
   begin
      ada.text_io.put(item => "K1: ");
      INTEGER_EA .put(item => ITEM.K1, width => 2);
      ada.text_io.put(item => "  K2: ");
      ada.text_io.put(item => ITEM.K2);
      ada.text_io.put(item => "  K3: ");
      INTEGER_EA .put(item => ITEM.K3, width => 2);
      ada.text_io.new_line;
   end PUT_VERBUND;
   ------------------------------------------------------------------------
   V_VAR:    V_TYP;              -- Zum Ausgeben und Einlesen von Verbunden
   AUSWEIS1: V_TYP_EA.file_type; -- Zugriffsberechtigung fuer Datei
   TEXT:     constant string(6..14) := "ABCDEFGHI"; -- Daten zum Fuellen
begin -- DARST_03
   ------------------------------------------------------------------------
   -- Die Groesse des Untertyps V_TYP und der Variablen V_VAR ausgeben:
   ada.text_io.put(item => "V_TYP'size: ");
   INTEGER_EA .put(item =>  V_TYP'size, width => 0);
   ada.text_io.new_line;
   ada.text_io.put(item => "V_VAR'size: ");
   INTEGER_EA .put(item =>  V_VAR'size, width => 0);
   ada.text_io.new_line;
   ------------------------------------------------------------------------
   -- Eine sequentielle Datei erstellen und 15 Verbunde hineinschreiben:
   V_TYP_EA.create(file => AUSWEIS1, name => "DARST_03.AUS");
   for I in natural range 8..10 loop
      for J in integer range -2..+2 loop
         V_VAR.K1 := I;
         V_VAR.K2 := TEXT(I+J..I+J+2);
         V_VAR.K3 := J;
         V_TYP_EA.write(file => AUSWEIS1, item => V_VAR);
      end loop;
   end loop;
   ------------------------------------------------------------------------
   -- Alle Verbunde aus der sequentielle Datei wieder einlesen und
   -- in Textform zur aktuellen Ausgabe ausgeben:
   V_TYP_EA.reset (file => AUSWEIS1, mode => V_TYP_EA.in_file);
   while not V_TYP_EA.end_of_file(file => AUSWEIS1) loop
      V_TYP_EA.read(file => AUSWEIS1, item => V_VAR);
      PUT_VERBUND(ITEM => V_VAR);
   end loop;
   ------------------------------------------------------------------------
   V_TYP_EA.close(file => AUSWEIS1);
end DARST_05;
