with ada.unchecked_deallocation, ada.unchecked_conversion, ada.text_io;
package body KON_B_01 is
   ------------------------------------------------------------------------
   procedure PUT(ITEM: ZEICHEN_KETTE; S: string) is
   -- Zum Protokollieren der Aufrufe der Prozeduren INITIALIZE, ADJUST und
   -- FINALIZE. Ausgegeben wird jeweils der String S, ein Zeiger, der auf
   -- ITEM zeigt, der Zeiger ITEM.Z und (falls der ungleich null ist)
   -- der String ITEM.Z.all.
      ---------------------------------------------------------------------
      -- Achtung: Beim Compiler Gnat 3.10 sind Zeiger vom Typ !Z_ZK nur
      -- 32 Bit lang, Zeiger vom Typ !Z_STRING dagegen 64 Bit! Bei anderen
      -- Compilern muss man die folgenden beiden Vereinbarungen anpassen.
      type    ADR_ZK    is mod 2**32; -- Zum Ausgeben von Z_ZK-Zeigern
      type    ADR_ST    is mod 2**64; -- Zum Ausgaben von Z-STRING-Zeigern
      package ADR_ZK_EA is new ada.text_io.modular_io(num => ADR_ZK);
      package ADR_ST_EA is new ada.text_io.modular_io(num => ADR_ST);
      
      function CONV1 is new
         ada.unchecked_conversion(source=>Z_ZK,     target=>ADR_ZK);
      function CONV2 is new
         ada.unchecked_conversion(source=>Z_STRING, target=>ADR_ST);

      ADR1 : ADR_ZK := CONV1(ITEM'unchecked_access); -- 'access geht nicht!
      ADR2 : ADR_ST := CONV2(ITEM.Z);
      B    : constant character := ' ';
   begin
      ada.text_io.put(item => B & S & B);
      ADR_ZK_EA.put(item => ADR1, BASE => 16);
      ADR_ST_EA.put(item => ADR2, BASE => 16);
      if ITEM.Z /= null then
         ada.text_io.put(item => " " & ITEM.Z.all);
      end if;
      ada.text_io.new_line;
   end PUT;
   ------------------------------------------------------------------------
   procedure GIB_ZURUECK is new ada.unchecked_deallocation
      (object => string, name => Z_STRING);
      -- Diese Prozedur hat einen in-out-Parmeter X vom Typ !Z_STRING.
      -- Das Objekt X.all wird an die Speicherverwaltung zurueckgegeben
      -- und dem Zeiger X wird der Wert null zugewiesen.
   ------------------------------------------------------------------------
   procedure INITIALIZE(ZK: in out ZEICHEN_KETTE) is
   -- Meldet nur ihre Aufrufe
   begin
      PUT(ITEM => ZK, S => "INI ");
   end INITIALIZE;
   ------------------------------------------------------------------------
   procedure ADJUST(ZK: in out ZEICHEN_KETTE) is
   -- Kopiert den String ZK.Z.all und laesst den Zeiger ZK.Z
   -- auf die Kopie zeigen.
   begin
      PUT(ITEM => ZK, S => "ADJ1");
      if ZK.Z /= null then
         ZK.Z := new string'(ZK.Z.all);
      end if;
      PUT(ITEM => ZK, S => "ADJ2");
   end ADJUST;
   ------------------------------------------------------------------------
   procedure FINALIZE(ZK: in out ZEICHEN_KETTE) is
   -- Gibt den String ZK.Z.all an die Speicherverwaltung zurueck.
   begin
      PUT(ITEM => ZK, S => "FIN1");
      GIB_ZURUECK(X => ZK.Z);
      PUT(ITEM => ZK, S => "FIN2");
   end FINALIZE;
   ------------------------------------------------------------------------
   procedure MACHE_ZEICHEN_KETTE(S: in string; ZK: in out ZEICHEN_KETTE) is
   -- Konstruktor-Prozedur fuer ZEICHEN_KETTEn
   begin
      PUT(ITEM => ZK, S => "MZK1");
      GIB_ZURUECK(X => ZK.Z);
      ZK.Z := new string'(S);
      PUT(ITEM => ZK, S => "MZK2");
   end MACHE_ZEICHEN_KETTE;
   ------------------------------------------------------------------------
   function MACHE_ZEICHEN_KETTE(S: in string) return ZEICHEN_KETTE is
   -- Konstruktor-Funktion fuer ZEICHEN_KETTEn. Vorsicht! Wenn man damit
   -- Variablen initialisiert, geht eventuell Speicher verloren!
      ZK : ZEICHEN_KETTE;
   begin
      ZK.Z := new string'(S);
      PUT(ITEM => ZK, S => "FMZ ");
      return ZK;
   end MACHE_ZEICHEN_KETTE;
   ------------------------------------------------------------------------
end KON_B_01;
