with ada.text_io;
procedure TASKS_07 is
---------------------------------------------------------------------------
-- Eine PRODUZENTen-Task und eine KONSUMENTen-Task sind ueber eine
-- LAGER-Task miteinander verbunden. Die LAGER-Task kann bestimmte
-- Geschwindigkeitsunterschiede zwischen dem Produzenten und dem
-- Konsumenten ausgleichen. Bei jeder Aenderung im LAGER wird die
-- aktuelle Anzahl der Kisten im Lager zur aktuellen Ausgabe ausgegeben.
---------------------------------------------------------------------------
   PRODUKT : string(1..10) := "ABCDEFGHIJ";
   package NATU_EA is new ada.text_io.integer_io(num => natural);
   ------------------------------------------------------------------------
   task PRODUZENT;
   task KONSUMENT;
   task LAGER is
      entry REIN(KISTE: in  character);
      entry RAUS(KISTE: out character);
   end LAGER;
   ------------------------------------------------------------------------
   task body PRODUZENT is
      P_ZEIT : array(PRODUKT'range) of duration := --  Produktionszeiten
      (0.1, 0.2, 0.1, 0.3, 0.2, 0.1, 0.0, 0.5, 0.1, 0.1);
      I : natural := PRODUKT'first;       -- Index fuer PRODUKT und P_ZEIT
   begin -- PRODUZENT
      loop
         delay P_ZEIT(I);                 -- Produktionszeit
         LAGER.REIN(KISTE => PRODUKT(I)); -- Produkt ins Lager
      exit when PRODUKT(I) = 'J';         -- letztes Produkt?
         I := I + 1;
      end loop;
   end PRODUZENT;
   ------------------------------------------------------------------------
   task body KONSUMENT is
      K_ZEIT : array(PRODUKT'range) of duration := -- Konsumptionszeiten
      (0.5, 0.4, 0.0, 0.0, 0.0, 0.5, 0.5, 0.8, 0.8, 0.0);
      I : natural := PRODUKT'first;       -- Index fuer PRODUKT und K_ZEIT
   begin -- KONSUMENT
      loop
         LAGER.RAUS(KISTE => PRODUKT(I)); -- Produkt aus dem Lager
         delay K_ZEIT(I);                 -- Konsumptionszeit
      exit when PRODUKT(I) = 'J';         -- letztes Produkt?
         I := I + 1;
      end loop;
   end KONSUMENT;
   ------------------------------------------------------------------------
   task body LAGER is
      MAX_ANZAHL : constant natural := 3; -- Kapazitaet des Lagers
      REGAL      : string(1..MAX_ANZAHL);
      AKT_ANZAHL : natural range 0..MAX_ANZAHL := 0;
      REIN_PLATZ : natural range 1..MAX_ANZAHL := 1;
      RAUS_PLATZ : natural range 1..MAX_ANZAHL := 1;
   begin  -- LAGER
      ada.text_io.put(item => "Lagerbestand: ");
      loop
         select
            when AKT_ANZAHL < MAX_ANZAHL =>
            accept REIN(KISTE: in character) do
               REGAL(REIN_PLATZ) := KISTE;
            end REIN;
            REIN_PLATZ := REIN_PLATZ mod MAX_ANZAHL + 1;
            AKT_ANZAHL := AKT_ANZAHL + 1;
            NATU_EA.put(item => AKT_ANZAHL, width => 2);
         or when AKT_ANZAHL > 0 =>
            accept RAUS(KISTE: out character) do
               KISTE := REGAL(RAUS_PLATZ);
            end RAUS;
            RAUS_PLATZ := RAUS_PLATZ mod MAX_ANZAHL + 1;
            AKT_ANZAHL := AKT_ANZAHL - 1;
            NATU_EA.put(item => AKT_ANZAHL, width => 2);
         or
            terminate;
         end select;
      end loop;
   end LAGER;
   ------------------------------------------------------------------------
begin -- TASKS_07
   null;
end TASKS_07;