--*******1*********2*********3*********4*********5*********6*********7*****
with ada.text_io;
with ada.characters.latin_1;
package body TST.TEXT_IO is
   -----------------------------------------------------------------------
   -- Allgemeine Hinweise zu diesem Paketrumpf:
   -- Alle GET-Prozeduren, die direkt oder indirekt in diesem Paket
   -- vereinbart werden (z.B. wird TST.TEXT_IO.GET direkt und
   -- TST.TEXT_IO.INTEGER_IO.GET indirekt vereinbart), lesen mithilfe
   -- der Prozedur LIES_EINE_ZEILE zunaechst Zeichenketten von der Stan-
   -- dardeingabe in die string-Variable ZEILE und verarbeiten sie dann
   -- weiter. Die Teilreihung ZEILE(VON..BIS) enthaelt stets alle
   -- Zeichen, die schon eingelesen aber noch nicht verarbeitet wurden.
   -- Nur die Prozedur LIES_EINE_ZEILE liest von der Standardeingabe.
   -- Die Prozedur LIES_EINE_ZEILE schreibt alles, was sie liest, unver-
   -- aendert in die Protokolldatei namens PROTOCOL.TMP (einschliesslich
   -- aller Zeilenwechsel).
   ------------------------------------------------------------------------
   -- Die Protokolldateien und ihre Namen: 
   PROTOKOLL     : ada.text_io.file_type;             -- Ein- und Ausgaben
   EINGABEN      : ada.text_io.file_type;             -- Nur Eingaben
   ------------------------------------------------------------------------
   -- Puffer zum Lesen von der Standardeingabe und zugehoerige Variablen:
   ZEILE : string(1..250) := (others => ' ');
   VON   : natural := ZEILE'first;
   BIS   : natural := ZEILE'first - 1;
   -- ZEILE(VON..BIS) enthaelt jeweils alle schon eingelesene, aber noch
   -- nicht verarbeiteten Zeichen.
   ------------------------------------------------------------------------
   procedure LIES_EINE_ZEILE(ITEM_L:       in out string;
                             VON_L, BIS_L: in out natural) is
      -- Liest von der Standardeingabe eine Zeile nach ITEM und gibt diese
      -- Zeile unveraendert in die PROTOKOLL-Datei aus.
      ---------------------------------------------------------------------
      VON_NEU : natural := ITEM_L'first;
      BIS_NEU : natural := ITEM_L'first + (BIS_L - VON_L);
   begin -- LIES_EINE_ZEILE
      -- Wenn die noch unverarbeiteten Zeichen ITEM(VON..BIS) nicht
      -- am Anfang von ITEM stehen, dann werden sie an den Anfang
      -- verschoben:
      if VON_L > ITEM_L'first then
         ITEM_L(VON_NEU..BIS_NEU) := ITEM_L(VON_L..BIS_L);
         VON_L := VON_NEU;
         BIS_L := BIS_NEU;
      end if;
      -- Eine Zeile von der Standardeingabe nach ITEM lesen und in die
      -- PROTOKOLL-Datei und in die EINGABEN-Datei ausgeben:
      ada.text_io.get_line(item => ITEM_L(BIS_L+1..ITEM_L'last),
         last => BIS_NEU);
      ada.text_io.put_line(item => ITEM_L(BIS_L+1..BIS_NEU),
         file => PROTOKOLL);
      ada.text_io.put_line(item => ITEM_L(BIS_L+1..BIS_NEU),
         file => EINGABEN);
      BIS_L := BIS_NEU;
   end LIES_EINE_ZEILE;      
   ------------------------------------------------------------------------
   procedure AUFFUELLEN1
      (ITEM_A  : in out string; VON_A, BIS_A: in out natural;
       WIDTH_A : in     field) is
      -- Liest solange weitere Zeilen nach ITEM_A, bis in ITEM_A
      -- mindestens WIDTH-viele Zeichen stehen:
      ---------------------------------------------------------------------
   begin
      -- (BIS_A - VON_A + 1) ist die Anzahl der schon eingelesenen aber 
      -- noch nicht verarbeiteten Zeichen
      while BIS_A - VON_A + 1 < natural(WIDTH_A) loop
         LIES_EINE_ZEILE(ITEM_L => ITEM_A, VON_L => VON_A, BIS_L => BIS_A);
      end loop;
   end AUFFUELLEN1;
   ------------------------------------------------------------------------
   procedure AUFFUELLEN2
      (ITEM_A: in out string; VON_A, BIS_A: in out natural) is
   -- Liest solange weitere Zeilen nach ITEM, bis in ITEM mindestens
   -- *ein* von BLANK und TAB verschiedenes Zeichen (ein NON_BOT) steht.
      function ANZ_NON_BOT(ITEM: string) return natural is
      -- Liefert die Anzahl der von BLANK und TAB verschiedenen Zeichen
      -- in ITEM.
         BLANK : constant character := ada.characters.latin_1.space;
         TAB   : constant character := ada.characters.latin_1.ht;
         ERG   :          natural   := 0;
      begin -- ANZ_NON_BOT
         for I in positive range ITEM'range loop
            if ITEM(I) /= BLANK and ITEM(I) /= TAB then
               ERG := ERG + 1;
            end if;
         end loop;
         return ERG;
      end ANZ_NON_BOT;
   begin -- AUFFUELLEN2
      while ANZ_NON_BOT(ITEM => ITEM_A(VON_A..BIS_A)) < 1 loop
         LIES_EINE_ZEILE(ITEM_L => ITEM_A, VON_L => VON_A, BIS_L => BIS_A);
      end loop;
   end AUFFUELLEN2;
   ------------------------------------------------------------------------
   -- Ein-/Ausgabe von einzelnen Zeichen vom Typ character:
   procedure GET     (ITEM: out character) is
   begin
      AUFFUELLEN1(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS, WIDTH_A => 1);
      ITEM := ZEILE(VON);
      VON  := VON + 1;
   end GET;

   procedure PUT     (ITEM: in  character) is
   begin
      ada.text_io.put(item => ITEM);
      ada.text_io.put(item => ITEM, file => PROTOKOLL);
   end PUT;

   -- Ein-/Ausgabe von Zeichenketten vom Typ string:
   procedure GET     (ITEM: out string) is
   begin
      AUFFUELLEN1(ITEM_A  => ZEILE, VON_A => VON , BIS_A => BIS,
                  WIDTH_A => ITEM'length);
      ITEM := ZEILE(VON..VON+ITEM'length-1);
      VON  := VON + ITEM'length;
   end GET;

   procedure GET_LINE(ITEM: out string; LAST: out natural) is
   begin
      -- Wenn sich in ZEILE keine unverarbeiteten Zeichen mehr befinden,
      -- wird eine neue Zeile eingelesen:
      if VON > BIS then
         LIES_EINE_ZEILE(ITEM_L => ZEILE, VON_L => VON, BIS_L => BIS);
      end if;
      ITEM(ITEM'first .. ITEM'first+(BIS-VON)) := ZEILE(VON .. VON+(BIS-VON));
      LAST := ITEM'first+(BIS-VON);
      VON  := ZEILE'first;
      BIS  := ZEILE'first - 1;
   end GET_LINE;

   procedure PUT     (ITEM: in  string) is
   begin
      ada.text_io.put(item => ITEM);
      ada.text_io.put(item => ITEM, file => PROTOKOLL);
   end PUT;

   procedure PUT_LINE(ITEM: in  string) is
   begin
      ada.text_io.put(item => ITEM);
      ada.text_io.new_line;
      ada.text_io.put(item => ITEM, file => PROTOKOLL);
      ada.text_io.new_line(         file => PROTOKOLL);
   end PUT_LINE;

   -- Neue Zeile anfangen:
   procedure NEW_LINE(SPACING: in positive_count := 1) is
   begin
      ada.text_io.new_line(spacing => SPACING);
      ada.text_io.new_line(spacing => SPACING, file => PROTOKOLL);
   end NEW_LINE;
   ------------------------------------------------------------------------
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von Ganzzahltypen: -----
   package body INTEGER_IO is
      package   NUM_EA is new ada.text_io.integer_io(num => NUM);
      procedure GET(ITEM: out NUM; WIDTH : in FIELD := 0) is
         BIS_HIER : natural;
      begin
         if WIDTH > 0 then -- genau WIDTH-viele Zeichen verarbeiten
            AUFFUELLEN1(ITEM_A  => ZEILE, VON_A => VON, BIS_A => BIS,
                        WIDTH_A => WIDTH);
            BIS_HIER := VON + natural(WIDTH) - 1;
         else
            AUFFUELLEN2(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS);
            BIS_HIER := BIS;
         end if;
         NUM_EA.get(from => ZEILE(VON..BIS_HIER), item => ITEM,
                    last => VON);
         VON := VON + 1;
      end GET;

      procedure PUT(ITEM: in  NUM; WIDTH : in FIELD := DEFAULT_WIDTH;
                                   BASE:   in NUMBER_BASE := DEFAULT_BASE) is
      begin
         NUM_EA.put(item => ITEM, width => width, base => BASE);
         NUM_EA.put(item => ITEM, width => width, base => BASE,
                    file => PROTOKOLL);
      end PUT;
   end INTEGER_IO;
   ------------------------------------------------------------------------
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von modularen Typen: ----
   package body MODULAR_IO is
      package   NUM_EA is new ada.text_io.modular_io(num => NUM);
      procedure GET(ITEM: out NUM; WIDTH : in FIELD := 0) is
         BIS_HIER : natural;
      begin
         if WIDTH > 0 then -- genau WIDTH-viele Zeichen verarbeiten
            AUFFUELLEN1(ITEM_A  => ZEILE, VON_A => VON, BIS_A => BIS,
                        WIDTH_A => WIDTH);
            BIS_HIER := VON + natural(WIDTH) - 1;
         else
            AUFFUELLEN2(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS);
            BIS_HIER := BIS;
         end if;
         NUM_EA.get(from => ZEILE(VON..BIS_HIER), item => ITEM,
                    last => VON);
         VON := VON + 1;
      end GET;

      procedure PUT(ITEM: in  NUM; WIDTH : in FIELD := DEFAULT_WIDTH;
                                   BASE:   in NUMBER_BASE := DEFAULT_BASE) is
      begin
         NUM_EA.put(item => ITEM, width => width, base => BASE);
         NUM_EA.put(item => ITEM, width => width, base => BASE,
                    file => PROTOKOLL);
      end PUT;
   end MODULAR_IO;
   ------------------------------------------------------------------------
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von Gleitpunkttypen: ----
   package body FLOAT_IO is
      package NUM_EA is new ada.text_io.float_io(num => NUM);
      procedure GET(ITEM: out NUM; WIDTH: in FIELD := 0) is
         BIS_HIER : natural;
      begin
         if WIDTH > 0 then -- genau WIDTH-viele Zeichen verarbeiten
            AUFFUELLEN1(ITEM_A  => ZEILE, VON_A => VON, BIS_A => BIS,
                        WIDTH_A => WIDTH);
            BIS_HIER := VON + natural(WIDTH) - 1;
         else
            AUFFUELLEN2(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS);
            BIS_HIER := BIS;
         end if;
         NUM_EA.get(from => ZEILE(VON..BIS_HIER), item => ITEM,
                    last => VON);
         VON := VON + 1;
      end GET;

      procedure PUT(ITEM: in  NUM; FORE : in FIELD := DEFAULT_FORE;
                                   AFT  : in FIELD := DEFAULT_AFT;
                                   EXP  : in FIELD := DEFAULT_EXP) is
      begin
         NUM_EA.put(item => ITEM, fore => FORE, aft => AFT, exp => EXP);
         NUM_EA.put(item => ITEM, fore => FORE, aft => AFT, exp => EXP,
                    file => PROTOKOLL);
      end PUT;
   end FLOAT_IO;
   ------------------------------------------------------------------------
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von gewoehnlichen ------- 
   -- Festpunkttypen:
   package body FIXED_IO is
      package NUM_EA is new ada.text_io.fixed_io(num => NUM);
      procedure GET(ITEM: out NUM; WIDTH: in FIELD := 0) is
         BIS_HIER : natural;
      begin
         if WIDTH > 0 then -- genau WIDTH-viele Zeichen verarbeiten
            AUFFUELLEN1(ITEM_A  => ZEILE, VON_A => VON, BIS_A => BIS,
                        WIDTH_A => WIDTH);
            BIS_HIER := VON + natural(WIDTH) - 1;
         else
            AUFFUELLEN2(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS);
            BIS_HIER := BIS;
         end if;
         NUM_EA.get(from => ZEILE(VON..BIS_HIER), item => ITEM,
                    last => VON);
         VON := VON + 1;
      end GET;
      procedure PUT(ITEM: in  NUM; FORE : in FIELD := DEFAULT_FORE;
                                   AFT  : in FIELD := DEFAULT_AFT;
                                   EXP  : in FIELD := DEFAULT_EXP) is
      begin
         NUM_EA.put(item => ITEM, fore => FORE, aft => AFT, exp => EXP);
         NUM_EA.put(item => ITEM, fore => FORE, aft => AFT, exp => EXP,
                    file => PROTOKOLL);
      end PUT;
   end FIXED_IO;
   ------------------------------------------------------------------------
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von dezimalen ----------- 
   -- Festpunkttypen:
   package body DECIMAL_IO is
      package NUM_EA is new ada.text_io.decimal_io(num => NUM);
      procedure GET(ITEM: out NUM; WIDTH: in FIELD := 0) is
         BIS_HIER : natural := BIS;
      begin
         if WIDTH > 0 then -- genau WIDTH-viele Zeichen verarbeiten
            AUFFUELLEN1(ITEM_A  => ZEILE, VON_A => VON, BIS_A => BIS,
                        WIDTH_A => WIDTH);
            BIS_HIER := VON + natural(WIDTH) - 1;
         else
            AUFFUELLEN2(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS);
            BIS_HIER := BIS;
         end if;
         NUM_EA.get(from => ZEILE(VON..BIS_HIER), item => ITEM,
                    last => VON);
         VON := VON + 1;
      end GET;

      procedure PUT(ITEM: in  NUM; FORE : in FIELD := DEFAULT_FORE;
                                   AFT  : in FIELD := DEFAULT_AFT;
                                   EXP  : in FIELD := DEFAULT_EXP) is
      begin
         NUM_EA.put(item => ITEM, fore => FORE, aft => AFT, exp => EXP);
         NUM_EA.put(item => ITEM, fore => FORE, aft => AFT, exp => EXP,
                    file => PROTOKOLL);
      end PUT;
   end DECIMAL_IO;
   ------------------------------------------------------------------------
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von diskreten Typen:
   package body ENUMERATION_IO is
      package ENUM_EA is new ada.text_io.enumeration_io(enum => ENUM);
      procedure GET(ITEM: out ENUM) is
      begin
         AUFFUELLEN2(ITEM_A => ZEILE, VON_A => VON, BIS_A => BIS);
         ENUM_EA.get(from => ZEILE(VON..BIS), item => ITEM,
                    last => VON);
         VON := VON + 1;
      end GET;

      procedure PUT(ITEM: in  ENUM; WIDTH: in FIELD    := DEFAULT_WIDTH;
                                     SET : in TYPE_SET := DEFAULT_SETTING) is
      begin
         ENUM_EA.put(item => ITEM, width => WIDTH, set => SET);
         ENUM_EA.put(item => ITEM, width => WIDTH, set => SET,
                     file => PROTOKOLL);
      end PUT;
   end ENUMERATION_IO;
   ------------------------------------------------------------------------
   procedure OPEN_ODER_CREATE(FILE: in out ada.text_io.file_type;
                              NAME: in     string) is
   -- Falls eine Datei namens NAME schon existiert, wird sie (mit dem
   -- Modus append_file) geoeffnet. Sonst wird eine neue Ausgabe-Datei
   -- namens NAME erzeugt.
   begin
      -- Falls moeglich, eine existierende Datei oeffnen:
      ada.text_io.open (file => FILE, mode => append_file, name => NAME);
   exception
      -- Falls noetig, eine neue Datei erzeugen:
      when ada.text_io.name_error =>
         ada.text_io.create(file => FILE, name => NAME);
   end OPEN_ODER_CREATE;
   ------------------------------------------------------------------------
begin -- TST.TEXT_IO
   OPEN_ODER_CREATE(FILE => PROTOKOLL, NAME => PROTOKOLL_NAM);
   OPEN_ODER_CREATE(FILE => EINGABEN,  NAME => EINGABEN_NAM);
end TST.TEXT_IO;
--*******1*********2*********3*********4*********5*********6*********7*****

