--*******1*********2*********3*********4*********5*********6*********7*****
with    ada.text_io;
package TST.TEXT_IO is
---------------------------------------------------------------------------
-- Ein *Standard-Dialog* besteht aus Eingaben von der Standardeingabe
-- und/oder aus Ausgaben zur Standardausgabe. Das Paket TST.TEXT_IO ist
-- eine Testhilfe fuer Programme, die Standard-Dialoge fuehren. Wenn man
-- das Paket TST.TEXT_IO anstelle des "normalen Paketes" ada.text_io ver-
-- wendet, dann werden die Standard-Dialoge in einer *Protokolldatei* 
-- festgehalten und die Eingaben von der Standardeingabe werden zusaetzlich
-- auch noch in eine *Eingaben-Datei* geschrieben. Mit diesen beiden 
-- Dateien lassen sich Testlaeufe leicht wiederholen:
-- Beim erstenmal gibt man die Testdaten manuell ueber die Standardeingabe
-- ein, beim Wiederholen des Testlaufs benuetzt man die Eingaben-Datei
-- als Standardeingabe (per Dateiumlenkung). In beiden Faellen muss die
-- gleiche Protokolldatei "herauskommen" (sonst wurde das zu testende Pro-
-- gramm zwischen den beiden Testlaeufen "wesentlich veraendert").
--
-- Nach einem Testlauf enthaelt die Protokolldatei alle Daten, die
-- waehrend des Testlaufs auf dem Bildschirm erschienen sind: nicht nur
-- die Ausgaben des zu testenden Programms, sondern auch die Eingaben des
-- Benutzers. In der Protokolldatei wird also ein genaues Abbild des
-- Bildschirms zusammengestellt.
--
-- Das "normale Paket" ada.text_io enthaelt GET- und PUT-Prozeduren
-- *ohne* file-Parameter ("Standard-GET- und Standard-PUT-Prozeduren")
-- und solche *mit* file-Parameter. TST.TEXT_IO enthaelt nur die
-- Standard-GET- und Standard-PUT-Prozeduren (die ohne file-Parameter).
-- Entsprechendes gilt auch fuer die generischen Pakete
-- TST.TEXT_IO.INTEGER_IO, TST.TEXT_IO.MODULAR_IO, TST.TEXT_IO.FLOAT_IO,
-- TST.TEXT_IO.FIXED_IO, TST.TEXT_IO.DECIMAL_IO und
-- TST.TEXT_IO.ENUMERATION_IO. Konsequenterweise gibt es in TST.TEXT_IO
-- auch keine OPEN-, CREATE- oder CLOSE-Prozeduren (die Standardeingabe
-- und die Standardausgabe sind immer geoeffnet und koennen nicht ge-
-- schlossen werden). 
---------------------------------------------------------------------------
   -- Die Datei-Namen der Protokoll-Datei und der Eingaben-Datei:
   PROTOKOLL_NAM : constant string := "PROTOCOL.TMP"; 
   EINGABEN_NAM  : constant string := "EINGABEN.TMP";
   -- Untertypen und andere Umbenennungen von Groessen aus ada.text_io:
   subtype  FIELD          is ada.text_io.field;
   subtype  NUMBER_BASE    is ada.text_io.number_base;
   subtype  COUNT          is ada.text_io.count;
   subtype  POSITIVE_COUNT is ada.text_io.positive_count;
   subtype  TYPE_SET       is ada.text_io.type_set;
   subtype  FILE_MODE      is ada.text_io.file_mode;
   function UPPER_CASE  return TYPE_SET  renames ada.text_io.upper_case;
   function LOWER_CASE  return TYPE_SET  renames ada.text_io.lower_case;
   function APPEND_FILE return FILE_MODE renames ada.text_io.append_file;
   -- Ein-/Ausgabe von einzelnen Zeichen vom Typ character:
   procedure PUT     (ITEM: in  character);
   procedure GET     (ITEM: out character);
   -- Ein-/Ausgabe von Zeichenketten vom Typ string:
   procedure PUT     (ITEM: in  string);
   procedure PUT_LINE(ITEM: in  string);
   procedure GET     (ITEM: out string);
   procedure GET_LINE(ITEM: out string; LAST: out natural);
   -- Neue Zeile anfangen: 
   procedure NEW_LINE(SPACING: in positive_count := 1);
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von Ganzzahltypen: -----
   generic
      type NUM is range <>;
   package INTEGER_IO is
      DEFAULT_WIDTH : FIELD := NUM'WIDTH;
      DEFAULT_BASE  : NUMBER_BASE := 10;
      procedure GET(ITEM: out NUM; WIDTH : in FIELD := 0);
      procedure PUT(ITEM: in  NUM; WIDTH : in FIELD := DEFAULT_WIDTH;
                                   BASE:   in NUMBER_BASE := DEFAULT_BASE);
   end INTEGER_IO;
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von modularen Typen: ----
   generic
      type NUM is mod <>;
   package MODULAR_IO is
      DEFAULT_WIDTH : FIELD := NUM'WIDTH;
      DEFAULT_BASE  : NUMBER_BASE := 10;
      procedure GET(ITEM: out NUM; WIDTH : in FIELD := 0);
      procedure PUT(ITEM: in  NUM; WIDTH : in FIELD := DEFAULT_WIDTH;
                                   BASE:   in NUMBER_BASE := DEFAULT_BASE);
   end MODULAR_IO;
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von Gleitpunkttypen: ----
   generic
      type NUM is digits <>;
   package FLOAT_IO is
      DEFAULT_FORE : FIELD := 2;
      DEFAULT_AFT  : FIELD := NUM'DIGITS-1;
      DEFAULT_EXP  : FIELD := 3;
      procedure GET(ITEM: out NUM; WIDTH: in FIELD := 0);
      procedure PUT(ITEM: in  NUM; FORE : in FIELD := DEFAULT_FORE;
                                   AFT  : in FIELD := DEFAULT_AFT;
                                   EXP  : in FIELD := DEFAULT_EXP);
   end FLOAT_IO;
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von gewoehnlichen ------- 
   -- Festpunkttypen:
   generic
      type NUM is delta <>;
   package FIXED_IO is
      DEFAULT_FORE : FIELD := NUM'FORE;
      DEFAULT_AFT  : FIELD := NUM'AFT;
      DEFAULT_EXP  : FIELD := 0;
      procedure GET(ITEM: out NUM; WIDTH: in FIELD := 0);
      procedure PUT(ITEM: in  NUM; FORE : in FIELD := DEFAULT_FORE;
                                   AFT  : in FIELD := DEFAULT_AFT;
                                   EXP  : in FIELD := DEFAULT_EXP);
   end FIXED_IO;
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von dezimalen ----------- 
   -- Festpunkttypen:
   generic
      type NUM is delta <> digits <>;
   package DECIMAL_IO is
      DEFAULT_FORE : FIELD := NUM'FORE;
      DEFAULT_AFT  : FIELD := NUM'AFT;
      DEFAULT_EXP  : FIELD := 0;
      procedure GET(ITEM: out NUM; WIDTH: in FIELD := 0);
      procedure PUT(ITEM: in  NUM; FORE : in FIELD := DEFAULT_FORE;
                                   AFT  : in FIELD := DEFAULT_AFT;
                                   EXP  : in FIELD := DEFAULT_EXP);
   end DECIMAL_IO;
   -- Gen. Paket fuer die Ein-/Ausgabe von Werten von diskreten Typen: ----
   generic
      type ENUM is (<>);  -- ENUM ist ein beliebiger diskreter Typ!
   package ENUMERATION_IO is
      DEFAULT_WIDTH   : FIELD := 0;
      DEFAULT_SETTING : TYPE_SET := UPPER_CASE;
      procedure GET(ITEM: out ENUM);
      procedure PUT(ITEM: in  ENUM; WIDTH: in FIELD    := DEFAULT_WIDTH;
                                     SET : in TYPE_SET := DEFAULT_SETTING);
   end ENUMERATION_IO;
   ------------------------------------------------------------------------      
end TST.TEXT_IO;
--*******1*********2*********3*********4*********5*********6*********7*****
   

