package TYPEN_04 is
---------------------------------------------------------------------------
-- Enthaelt Beispiele fuer "alle moeglichen" Typvereinbarungen. Von jedem
-- Typ wird angegeben, zu welchen vordefinierten Typklassen er gehoert
-- (z.B. Aufzaehlungstypen, diskrete Typen, skalare Typen etc.). Ausserdem
-- werden wichtige Eigenchaften der Typen und ihrer ersten Untertypen
-- genannt (z.B. eingeschraenkt/uneingeschraenkt, definit/indefinit etc.).
---------------------------------------------------------------------------
-- Zeichentypen (Aufzaehlungs-, diskrete, skalare, elementare Typen)
-- Muessen mindestens ein Zeichenliteral enthalten:

type ZEICHEN1 is ('A', 'B', 'C', 'a', 'b', 'c');
type ZEICHEN2 is ('a', BEH, CEH);

-- Die Untertypen ZEICHEN1 und ZEICHEN2 sind beide eingeschraenkt und
-- definit.
---------------------------------------------------------------------------
-- Boolesche Typen (Aufzaehlungs-, diskrete, skalare, elementare Typen)
-- Muessen direkt odr indirekt vom Typ boolean abgeleitet sein:

type BOOL1 is new boolean;
type BOOL2 is new BOOL1;

-- Die Untertypen BOOL1 und BOOL2 sind beide eingeschraenkt und definit
---------------------------------------------------------------------------
-- Andere Aufzaehlungstypen (Aufzaehlungs-, diskrete, skalare, elementare T.)
-- Duerfen kein Zeichenliteral enthalten:

type AUFZAEHL1 is (ROT, GRUEN, BLAU);

-- Der Untertyp AUFZAEHL1 ist eingeschraenkt und definit.
---------------------------------------------------------------------------
-- Signierte Ganzzahltypen (Ganzzahl-, diskrete, skalare, elementare Typen)
-- Koennen positive und negative Werte umfassen:

type GANZ1 is range -5_000..+10_000;
type GANZ2 is range   1900..2100;

-- Die Untertypen GANZ1 und GANZ2 sind eingeschraenkt und definit.
---------------------------------------------------------------------------
-- Modulare Ganzzahltypen (Ganzzahl-, diskrete, skalare, elementare Typen)
-- Koennen nur nichtnegative Werte enthalten:

type MODU1 is mod 256; -- Werte 0..255, 255 + 1 ist gleich 0.
type MODU2 is mod 12;  -- Werte 0..11,   11 + 1 ist gleich 0.

-- Die Untertypen MODU1 und MODU2 sind eingeschraenkt und definit.
---------------------------------------------------------------------------
-- Gleitpunkttypen (Bruchzahl-, skalare, elementare Typen)
-- Ein Bereich kann angegeben werden, musss aber nicht:

type GLEIT1 is digits 5;
type GLEIT2 is digits 5 range -100.0..+200.0;

-- Die Untertypen GLEIT1 und GLEIT2 sind beide definit, aber
-- GLEIT1 ist uneingeschraenkt (!) und GLEIT2 ist eingeschraenkt.
---------------------------------------------------------------------------
-- Gewoehnliche Festpunkttypen (Festpunkt-, Bruchzahl-, skalare, elem. T.)
-- Die delta-Zahl darf eine beliebige Bruchzahl sein, die small-Zahl wird
-- vom Ausfuehrer gewaehlt, ein Bereich muss angegeben werden:

type FEST1 is delta 1.0 / 16.0 range         -100.0..+200.0;
type FEST2 is delta    0.001   range         -200.0..+100.0;
type FEST3 is delta 1000.0     range -100_000_000.0..+200_000_000.0;

-- Die Untertypen FEST1 bis FEST3 sind eingeschraenkt und definit.
---------------------------------------------------------------------------
-- Dezimale Festpunkttypen (Festpunkt-, Bruchzahl-, skalare, elem. Typen)
-- Die delta-Zahl muss eine Zehnerpotenz sein, die small-Zahl ist gleich
-- der delta-Zahl. Ein Bereich kann angegeben werden, muss aber nicht:

type DEZ1 is delta 0.01 digits 8;
type DEZ2 is delta 0.01 digits 8 range -300_000.00..+500_000.00;

-- Die Untertypen DEZ1 und DEZ2 sind eingeschraenkt und definit.
---------------------------------------------------------------------------
-- Objekt-Zeigertypen (Zeiger-, elementare Typen)
-- Jeder Objekt-Zeigertyp hat einen Zieluntertyp und seine (Zeiger-) Werte
-- koennen nur auf Objekte dieses Untertyps zeigen.

type OB_ZEIG1 is access          DEZ1;   -- Zieluntertyp DEZ1
type OB_ZEIG2 is access all      DEZ1;   -- Zieluntertyp DEZ1
type OB_ZEIG3 is access constant DEZ1;   -- Zieluntertyp DEZ1
type OB_ZEIG4 is access          string; -- Zieluntertyp string
type OB_ZEIG5 is access all      string; -- Zieluntertyp string
type OB_ZEIG6 is access constant string; -- Zieluntertyp string

-- Objekte der Untertypen OB_ZEIG1 bis OB_ZEIG3 koennen zeigen auf:
-- OB_ZEIG1: Nur auf allokierte Variablen.
-- OB_ZEIG2: Auf allokierte und auf vereinbarte Alias-Variablen.
-- OB_ZEIG3: Auf allokierte Variablen und auf vereinbarte Alias-Variablen
--           und Alias-Konstanten. Keine Schreibberechtigung!
-- Fuer OB_ZEIG4 bis OB_ZEIG6 ganz entsprechend.
-- Die Untertypen OB_ZEIG1 bis OB_ZEIG6 sind definit.
-- OB_ZEIG1 bis OB_ZEIG3 sind eingeschraenkt (weil DEZ1 eingechraenkt ist).
-- OB_ZEIG3 bis OB_ZEIG6 sind uneingeschraenkt (weil string uneing. ist).
---------------------------------------------------------------------------
-- Unterprogramm-Zeigertypen (Zeiger-, elementare Typen)
-- Jeder Unterprogramm-Zeigertyp hat ein Zielprofil und seine (Zeiger-)
-- Werte koennen nur auf Unterprogramme mit diesem Profil zeigen.

type UP_ZEIG1 is access procedure          (I: in out integer);
type UP_ZEIG2 is access function           (I: in     integer) return float;
type UP_ZEIG3 is access protected procedure(I: in out integer);
type UP_ZEIG4 is access protected function (I: in     integer) return float;

-- Die (Zeiger-) Werte der Untertpen UP_ZEIG3 und UP_ZEIG4 koennen nur auf
-- entsprechende Unterprogramme eines geschuetzten Typs zeigen (s.u.). Die
-- Werte der Untertypen UP_ZEIG1 und UP_ZEIG2 koennen nur auf entsprechende
-- "normale" Unterprogramme zeigen.
-- Die Untertypen UP_ZEIG1 bis UP_ZEIG4 sind eingeschraenkt und definit.
---------------------------------------------------------------------------
-- Zeichenkettentypen (Reihungs-, zusammengesetzte Typen)
-- Eindimensionale Reihungstypen mit einem Zeichentyp als Komponententyp.
-- Fuer diese Typen gibt es Zeichenkettenliterale.

type KET1 is array(boolean) of ZEICHEN1;
type KET2 is array(boolean) of character;
type KET3 is array(positive range <>) of ZEICHEN2;
type KET4 is array(GANZ1    range <>) of character;

-- Die Untertypen KET1 und KET2 sind   eingeschraenkt und   definit.
-- Die Untertypen KET3 und KET4 sind uneingeschraenkt und indefinit.
---------------------------------------------------------------------------
-- Andere Reihungstypen (Reihungs-, zusammengesetzte Typen)
-- Ein n-dimensionaler Reihungstyp hat n Indexuntertypen und einen  Kompo-
-- nentenuntertyp:

type REIH1 is array(character)                            of boolean;
type REIH2 is array(character, boolean)                   of float;
type REIH3 is array(positive range <>)                    of float;
type REIH4 is array(positive range <>, positive range <>) of float;
type REIH5 is array(positive range <>, boolean  range <>) of REIH2;

-- Die Typen !REIH1 und !REIH3 sind eindimensional.
-- !REIH2, REIH4 und REIH5 sind zweidimensional.
--Die Untertypen REIH1 und REIH2 sind   eingeschraenkt und   definit.
--Die Untertypen REIH3 bis REIH5 sind uneingeschraenkt und indefinit.
---------------------------------------------------------------------------
-- Nicht-etikettierte Verbundtypen (Verbund-, zusammengesetzte Typen)
-- Ein Verbunduntertyp kann Diskriminanten haben, muss aber nicht:

type VERB1 is record
   K1: boolean;
   K2: integer;
   K3: integer;
end record;

type VERB2(D1: character) is record
   K1: boolean;
   K2: string(1..8);
end record;

type VERB3(D1: character := 'a') is record
   K1: boolean;
   K2: string(1..8);
end record;

type VERB4(D1, D2: natural; D3: boolean) is record
   K1: VERB1;
   K2: float;
end record;

type VERB5(D1, D2: natural := 0; D3: boolean := true) is record
   K1: VERB1;
   K2: float;
end record;

type VERB6 is limited record
  K1: boolean;
  K2: integer;
end record;

-- Wichtige Eigenschaften der Untertypen VERB1 bis VERB6:
-- VERB1: Keine Diskriminanten,   eingeschraenkt,   definit.
-- VERB2: Eine  Diskriminante,  uneingeschraenkt, indefinit.
-- VERB3: Eine  Diskriminante,  uneingeschraenkt,   definit!
-- VERB4: Drei  Diskriminanten, uneingeschraenkt, indefinit.
-- VERB5: Drei  Diskriminanten, uneingeschraenkt,   definit!
-- VERB6: Wie VERB5, aber limitiert.
---------------------------------------------------------------------------
-- Etikettierte Verbundtypen (Verbund-, zusammengesetzte Typen)
-- Koennen erweitert werden und bilden Ausgangspunkte fuer klassenweite T.:

type ETKET01 is tagged null record;           -- Null Komponenten

type ETKET02 is tagged record                 -- Zwei Komponenten
   K1: boolean;
   K2: integer;
end record;

type ETKET03 is new ETKET01 with record       -- Erweiterung von ETKET01
   K1: boolean;
   K2: integer;
end record;

type ETKET04 is new ETKET02 with record       -- Erweiterung von ETKET02
   K3: float;
   K4: float;
end record;

type ETKET05 is new ETKET04 with null record; -- Erweiterung von ETKET04

-- Nur-Vater-Typen (abstract types), limitierte Typen, private Typen und
-- Typen mit bekannten Diskriminanten:

type ETKET06 is abstract tagged record        -- Nur-Vater-Typ
  K1: boolean;                                -- ("abstract type")
  K2: integer;
end record;

type ETKET07 is tagged limited record         -- Limitierter Typ
   K1: boolean;
   K2: integer;
end record;

type ETKET08 is tagged private;               -- Privater Typ

type ETKET09(D1: character) is tagged record  -- Mit bekannten Diskrimi.
   K1: boolean;
   K2: integer;
end record;

-- Die vier Eigenschaften 1. Nur-Vater ("abstrakt"), 2. limitiert,
-- 3. privat und 4. mit-bekannten-Diskriminanten koennen beliebig
-- kombiniert werden. Der folgende Typ hat alle vier Eigenschaften:

type ETKET10(D1: character) is abstract tagged limited private;

-- Private etikettierte Verbundtypen (vereinbart mit "tagged private",
-- "tagged limited private", "abstract tagged private" oder "abstract
-- tagged limited private") koennen statt mit bekannten Diskriminanten
-- oder ohne Diskriminanten auch mit unbekannten Diskriminanten
-- vereinbart werden, z.B. so:

type ETKET11(<>) is tagged private; -- Volle Vereinbarung ohne Diskrimi.
type ETKET12(<>) is tagged private; -- Volle Vereinbarung mit  Diskrimi.

-- Wichtige Eigenschaften der Untertypen ETKET01 bis ETKET21:
-- ETKET01: null  Komponenten,   eingeschraenkt,   definit.
-- ETKET02: zwei  Komponenten,   eingeschraenkt,   definit.
-- ETKET03: zwei  Komponenten,   eingeschraenkt,   definit.
-- ETKET04: vier  Komponenten,   eingeschraenkt,   definit.
-- ETKET05: vier  Komponenten,   eingeschraenkt,   definit.

-- ETKET06: Wie ETKET02, aber nur-Vater-Typ ("abstrakt").
-- ETKET07: Wie ETKET02, aber limitiert.
-- ETKET08: privat,                           eingeschraenkt,   definit.
-- ETKET09: Mit bekannter   Diskriminante,  uneingeschraenkt, indefinit.
-- ETKET10: Mit bekannter   Diskriminante,  uneingeschraenkt, indefinit
--          Nur-Vater-Typ, limitiert, privat.
-- ETKET11: Mit unbekannten Diskriminanten, uneingeschraenkt, indefinit.
-- ETKET12: Mit unbekannten Diskriminanten, uneingeschraenkt, indefinit.

-- Klassenweite Untertypen wie ETKET1'class, ETKET2'class etc.
-- sind uneingeschraenkt und indefinit.
---------------------------------------------------------------------------
-- Private Typen: Sie koennen nur in der Spezifikation eines Paketes ver-
-- einbart werden und bilden keine eigene Klasse. Ein privter Typ reprae-
-- sentiert nur eine bestimmte Sicht ("eine Teilansicht") auf einen Typ, der
-- im privaten Teil der Paketspezifikation "voll vereinbart" werden muss.
-- Beispiel fuer etikettierte private Typen siehe vorigen Abschnitt.

type PT01                                             is         private;
type PT02(D1: character)                              is         private;
type PT03(D1, D2: natural := 0; D3: boolean := false) is         private;
type PT04(<>)                                         is         private;
type PT05(<>)                                         is         private;

type PT06                                             is limited private;
type PT07(D1: character)                              is limited private;
type PT08(D1, D2: natural := 0; D3: boolean := false) is limited private;
type PT09(<>)                                         is limited private;
type PT10(<>)                                         is limited private;


-- Wichtige Eigenschaften der (Unter-) Typen PT01 bis PT10:
-- PT01: nicht-limit.,   eingeschraenkt,   definit, keine Diskriminanten.
-- PT02: nicht-limit., uneingeschraenkt, indefinit, bekannte   Diskrimi.
-- PT03: nicht-limit., uneingeschraenkt,   definit, bekannte   Diskrimi.
-- PT04: nicht-limit., uneingeschraenkt, indefinit, unbekannte Diskrimi.
-- PT05: nicht-limit., uneingeschraenkt, indefinit, unbekannte Diskrimi.

-- PT06: limitiert,      eingeschraenkt,   definit, keine Diskriminanten.
-- PT07: limitiert,    uneingeschraenkt, indefinit, bekannte   Diskrimi.
-- PT08: limitiert,    uneingeschraenkt,   definit, bekannte   Diskrimi.
-- PT09: limitiert,    uneingeschraenkt, indefinit, unbekannte Diskrimi.
-- PT10: limitiert,    uneingeschraenkt, indefinit, unbekannte Diskrimi.
---------------------------------------------------------------------------
-- Generisch formale Typen (GFT) ("Schablonenparameter") und ihre
-- Eigenschaften:

generic
   type GFT11 is       (<>);          -- Beliebiger diskreter Typ.
   type GFT12 is range  <>;           -- Signierter Ganzzahltyp.
   type GFT13 is mod    <>;           -- Modularer  Ganzzahltyp.
   type GFT14 is digits <>;           -- Gleitpunkttyp.
   type GFT15 is delta  <>;           -- Gewoehnlicher Festpunkttyp.
   type GFT16 is delta  <> digits <>; -- Dezimaler     Festpunkttyp.
function SCHABLONE1(P: integer) return string;

-- Alle formalen Parametertypen GFT11 bis GFT16 sind eingeschraenkt und
-- definit.

generic
   type GFT21 is array(character)          of boolean; -- Reihungstyp
   type GFT22 is array(character range <>) of boolean; -- Reihungstyp
function SCHABLONE2(P: integer) return string;

-- Der formale Parametertyp GFT21 ist eingeschraenkt und definit,
-- dagegen ist GFT22 uneingeschraenkt und indefinit.

generic
   type GFT31 is access          integer; -- Zeigertyp mit Zieltyp integer
   type GFT32 is access          integer; -- Zeigertyp mit Zieltyp integer
   type GFT33 is access          integer; -- Zeigertyp mit Zieltyp integer
   type GFT34 is access          string;  -- Zeigertyp mit Zieltyp string
   type GFT35 is access all      string;  -- Zeigertyp mit Zieltyp string
   type GFT36 is access constant string;  -- Zeigertyp mit Zieltyp string
function SCHABLONE3(P: integer) return string;

-- Alle formalen Parametertypen GFT31 bis GFT36 sind definit.
-- Die Typen GFT31 bis GFT33 sind eingeschraenkt (weil ihr Zieltyp
-- integer eingeschraenkt ist), dagegen sind GFT34 bis GFT36 uneinge-
-- schraenkt (weil ihr Zieltyp string uneingeschraenkt ist).

generic
   type GFT41     is                private;
   type GFT42     is        limited private;
   type GFT43     is tagged         private;
   type GFT44     is tagged limited private;
   type GFT45(<>) is                private;
   type GFT46(<>) is        limited private;
   type GFT47(<>) is tagged         private;
   type GFT48(<>) is tagged limited private;
function SCHABLONE4(P: integer) return string;

-- GFT41: Ein   definiter,                nicht-limitierter Typ.
-- GFT42: Ein   definiter                                   Typ.
-- GFT43: Ein   definiter, etikettierter, nicht-limitierter Typ. 
-- GFT44: Ein   definiter, etikettierter                    Typ.
-- GFT45: Ein indefiniter,                nicht-limitierter Typ.
-- GFT46: Ein indefiniter,                                  Typ.
-- GFT47: Ein indefiniter, etikettierter, nicht-limitierter Typ.
-- GFT48: Ein indefiniter, etikettierter                    Typ.

generic
   type GFT51     is          new natural;
   type GFT52     is          new string;
   type GFT53     is          new ETKET01 with private;
   type GFT54     is abstract new ETKET01 with private;
   type GFT55     is          new ETKET10 with private;
   type GFT56     is abstract new ETKET10 with private;
   type GFT57(<>) is          new ETKET01 with private;
   type GFT58(<>) is abstract new ETKET01 with private;
function SCHABLONE5(P: integer) return string;

-- GFT51:   Definit (weil natural   definit ist).
-- GFT52: Indefinit (weil string  indefinit ist).
-- GFT53:   Definit (weil ETKET01   definit ist).
-- GFT54:   Definit (weil ETKET01   definit ist),           nur-Vater-Typ.
-- GFT55: Indefinit (weil ETKET10 indefinit ist).
-- GFT56: Indefinit (weil ETKET10 indefinit ist),           nur-Vater-Typ.
-- GFT57: Indefinit (wegen der unbekannten Diskriminanten)
-- GFT58: Indefinit (wegen der unbekannten Diskriminanten), nur-Vater-Typ
---------------------------------------------------------------------------
private
   type ETKET08 is tagged record
      K1: boolean;
      K2: integer;
   end record;
   
   type ETKET10(D1: character) is abstract tagged limited record
      K1: boolean;
      K2: integer;
   end record;
   
   type ETKET11 is tagged record                 -- Ohne Diskriminante
      K1: boolean;
      K2: integer;
   end record;
   
   type ETKET12(D1: character) is tagged record  -- Mit  Diskriminante
      K1: boolean;
      K2: integer;
   end record;

   type PT01 is range 0..100;

   type PT02(D1: character) is record
      K1: boolean;
      K2: integer;
   end record;

   type PT03(D1, D2: natural := 0; D3: boolean := false) is record
      K1: boolean;
      K2: integer;
   end record;
   
   type PT04 is range 0..100;                 -- Ohne Diskriminante.
   
   type PT05(D1: character) is record         -- Mit  Diskriminante.
      K1: boolean;
   end record;
   
   type PT06 is (ROT, GRUEN, BLAU);
   
   type PT07(D1: character) is record
      K1: boolean;
      K2: integer;
   end record;
   
   type PT08(D1, D2: natural := 0; D3: boolean := false) is record
      K1: boolean;
      K2: integer;
   end record;
   
   type PT09 is range 0..100;                 -- Ohne Diskriminante.
   
   type PT10(D1: character) is record         -- Mit  Diskriminante.
      K1: boolean;
   end record;   



end TYPEN_04;
