package TYPEN is
   -- 1. Elementare Typen
   -- 1.1. Skalare Typen
   -- 1.1.1. Diskrete Typen
   -- 1.1.1.1. Aufzaehlungstypen
   ------------------------------------------------------------------------
   -- 1.1.1.1.1. Zeichentypen (mindestens ein Zeichenliteral)
   type VOKAL     is ('a', 'e', 'i', 'o', 'u', MURMEL, PSST);
   ------------------------------------------------------------------------
   -- 1.1.1.1.2. boolesche Typen 
   type W_WERT    is new boolean;
   type T_VALUE   is new W_WERT;
   ------------------------------------------------------------------------
   -- 1.1.1.1.3. Andere Aufzaehlungstypen
   type FACH      is (MATHE, BIO, LATEIN, SPORT, MUSIK);
   type ZENSUR    is (SEHR_GUT, GUT, BEFRIEDIGEND, AUSREICHEND,
                      MANGELHAFT, UNGENUEGEND);
   type WAGEN_ART is (PERSONEN, LAST);
   ------------------------------------------------------------------------
   -- 1.1.1.2 Ganzzahl-Typen
   ------------------------------------------------------------------------
   -- 1.1.1.2.1. Signierte ("vorzeichenbehaftete") Ganzzahltypen.
   type ANZAHL    is range -100..+200;
   ------------------------------------------------------------------------
   -- 1.1.1.2.2. Modulare Ganzzahltypen
   type MONATS_NR is mod 12; -- Werte: 0..11, 11 + 1 ist gleich 0
   type BITS16    is mod 2**16;
   ------------------------------------------------------------------------
   -- 1.1.2. Bruchzahl-Typen
   ------------------------------------------------------------------------
   -- 1.1.2.1. Gleitpunkttypen (ein Bereich kann angegeben werden)
   type GLEIT1    is digits 6;
   type GLEIT2    is digits 6 range -100.0 .. +200.0;
   ------------------------------------------------------------------------
   -- 1.1.2.2. Festpunkttypen
   ------------------------------------------------------------------------
   -- 1.1.2.2.1. Gewoehnliche Festpunktt. (ein Bereich muss angegeb. werd.)
   type FEST1     is delta 0.01 range -100.0 .. +200.0;
   ------------------------------------------------------------------------
   -- 1.1.2.2.2. Dezimale Festpunkttypen (ein Bereich kann angegeben werden)
   type FEST2     is delta 0.01 digits 7;
   type FEST3     is delta 0.01 digits 7 range -50_000.0 .. +90_000.0;
   ------------------------------------------------------------------------
   -- 1.2. Zeiger-Typen
   ------------------------------------------------------------------------
   -- 1.2.1. Objekt-Zeigertypen
   type OB_ZEIG1  is access          FEST3;
   type OB_ZEIG2  is access all      FEST3;
   type OB_ZEIG3  is access constant FEST3;
   ------------------------------------------------------------------------
   -- 1.2.2. Unterprogramm-Zeigertypen
   type UP_ZEIG1  is access procedure(P1: in boolean; P2: in out integer);
   type UP_ZEIG2  is access function (P : integer) return boolean;
   type UP_ZEIG3  is access protected procedure(P: in out integer);
   type UP_ZEIG4  is access protected function (P: integer) return integer;
   ------------------------------------------------------------------------
   -- 2. Zusammengesetzte Typen
   -- 2.1. Reihungstypen
   ------------------------------------------------------------------------
   -- 2.1.1. Eingeschraenkte Reihungs-Typen
   type ZEUGNIS   is array(FACH)                   of ZENSUR;
   type Z_HEFT    is array(MONATS_NR)              of ZEUGNIS;
   type DIM_3     is array(boolean, FACH, boolean) of GLEIT1;
   ------------------------------------------------------------------------
   -- 2.1.2. Uneingeschraenkte Reihungs-Typen
   type ZEUGNIS_U is array(FACH      range <>)                of ZENSUR;
   type Z_HEFT_U  is array(MONATS_NR range <>)                of ZEUGNIS;
   type DIM_2_U   is array(boolean   range <>, FACH range <>) of GLEIT1;
   ------------------------------------------------------------------------
   -- 2.2. Nicht-etikettierte Verbundtypen
   ------------------------------------------------------------------------
   -- 2.2.1. Nicht-ettiketierte Verbundtypen ohne Diskriminanten/Varianten
   type LASTWAGEN is record
      GEWICHT : ANZAHL             := 0;
      KENNZ   : string(1..10)      := "??????????";
      ACHSEN  : ANZAHL range 0..20 := 0;
      HAENGER : boolean            := false;
   end record;
   ------------------------------------------------------------------------
   -- 2.2.2. Nicht-ettiketierte Verbundtypen mit Diskriminanten/Varianten   
   type WAGEN(ART: WAGEN_ART) is record
      GEWICHT : ANZAHL             := 0;
      KENNZ   : string(1..10)      := "!!!!!!!!!!";
      case ART is
         when PERSONEN =>
            SITZE  : ANZAHL range 0..10 := 0;
         when LAST     =>
            ACHSEN : ANZAHL range 0..20 := 0;
            HAENGER: boolean            := false;
      end case;
   end record;

   type FLEX_ZEUGNIS(VON, BIS: FACH := MUSIK) is record
      ZH: ZEUGNIS_U(VON..BIS);
   end record;
   ------------------------------------------------------------------------
   -- 2.3. Etikettierte Verbund-Typen
   type PUNKT is tagged record
      X, Y     : float   := 0.0;
      SICHTBAR : boolean := false;
   end record;

   type DINGSBUMS is tagged null record;

   type POINT is new PUNKT with null record;

   type PUNKT_MIT_FACH is new PUNKT with record
      F: FACH    := MUSIK;
      B: boolean := true;
   end record;

   type GLEIT_DING is new DINGSBUMS with record
      G: GLEIT1;
   end record;

   type FEST_DING  is new DINGSBUMS with record
      F: FEST1;
   end record;
   ------------------------------------------------------------------------
   -- 2.4. Task-Typen
   task type ZEICHEN_PUFFER is             -- ohne Diskriminanten
      entry TUE_REIN(C: in  character);
      entry HOL_RAUS(C: out character);
      entry LOESCHE_ALLE_ZEICHEN;
   end ZEICHEN_PUFFER;

   task type AUSGABE(NAME: character) is   -- mit einer Diskriminanten
      entry EINGANG1;
   end AUSGABE;
   ------------------------------------------------------------------------
   -- 2.5. Geschuetzte Typen
   protected type GESCHUETZTES_ZEUGNIS is  -- ohne Diskriminante
      function  LIES            (F: in FACH) return ZENSUR;
      procedure TRAGE_ZENSUR_EIN(F: in FACH; Z: in ZENSUR);
   private
      Z: ZEUGNIS := (others => SEHR_GUT);
   end GESCHUETZTES_ZEUGNIS;

   protected type ZAHL(NAME: character) is -- mit Diskriminante
      function  LIES;
      procedure SCHREIB(WERT: in integer);
   private
      Z: integer := 0;
   end GESCHUETZTE_ZAHL;
end TYPEN;   