with ada.text_io;
procedure UPROS_26 is
---------------------------------------------------------------------------
-- Eine Divisionsoperation namens "&" fuer Ganzzahlen wird vereinbart. 
-- Diese Operation "rundet" auch bei negativen Ergebnissen immer ab (die
-- vordefinierte Divisionsoperation "/" rundet immer zur 0 hin, bei nega-
-- tiven Ergebnissen also auf). Zwei Ganzzahlen werden eingelesen, einige
-- Ergebnisse der Operationen "/", "rem", "&" und "mod" werden ausgegeben.
---------------------------------------------------------------------------
   type    GANZ    is range -999..+999;
   package GANZ_EA is new ada.text_io.integer_io(num => GANZ);
   G1, G2: GANZ;
   UEBERSCHRIFTEN: constant string :=
      "  G1  G2  G1/G2  G1 rem G2  G1&I2  G1 mod G2  P1  P2";
   ------------------------------------------------------------------------
   function "&"(LINKS, RECHTS: GANZ) return GANZ is
   -- Eine Ganzzahldivision, die immer abrundet, auch bei negativen
   -- Ergebnissen, z.B. so:
   -- +7 & +3 ist gleich 2, aber -7 & +3 ist gleich -3
   ---------------------------------------------------
      QUOTIENT            : constant GANZ    := LINKS  /  RECHTS;
      REST                : constant GANZ    := LINKS rem RECHTS;
      UNGLEICHE_VORZEICHEN: constant boolean :=
         (LINKS < 0) /= (RECHTS < 0);
   begin
      if UNGLEICHE_VORZEICHEN and then REST /= 0 then
         return QUOTIENT - 1;
      else
         return QUOTIENT;
      end if;
   end "&";
   ------------------------------------------------------------------------
begin
   ------------------------------------------------------------------------
-- -- Den Zusammenhang zwischen der "&"-Division und dem mod-Rest 
-- -- systematisch testen:
-- for I1 in GANZ range -100..+100 loop
--    for I2 in GANZ range -100..+100 loop
--       if I2 /= 0 then
--          pragma assert(I1 = (I1&I2) * I2 + (I1 mod I2));
--          null;
--       end if;
--    end loop;
-- end loop;
   ------------------------------------------------------------------------
   loop
      ada.text_io.put("Eine Ganzzahl G1 (-999..+999)?                ");
      GANZ_EA    .get(G1);
      ada.text_io.put("Eine Ganzzahl G2 (-999..+999, 0 zum Beenden)? ");
      GANZ_EA    .get(G2);
   exit when G2 = 0;
      ada.text_io.new_line;
      ada.text_io.put_line(item => UEBERSCHRIFTEN);
      for I1 in GANZ range G1..G1+9 loop  
         GANZ_EA.put(item => I1);
         GANZ_EA.put(item => G2);
         GANZ_EA.put(item => I1  /  G2, width =>  7); -- Die alte Division!
         GANZ_EA.put(item => I1 rem G2, width => 11);
         GANZ_EA.put(item => I1  &  G2, width =>  7); -- Die neue Division!
         GANZ_EA.put(item => I1 mod G2, width => 11);
         GANZ_EA.put(item => I1 -((I1/G2)*G2 + I1 rem G2));
         GANZ_EA.put(item => I1 -((I1&G2)*G2 + I1 mod G2));
         ada.text_io.new_line;
      end loop;
   end loop;
   ada.text_io.new_line;
   ada.text_io.put_line(item => "UPROS_25: Das war's erstmal!");
end UPROS_26;