with ada.text_io;
procedure VERBU_02 is
---------------------------------------------------------------------------
-- Rechnet mit "Zahlen mit Einheiten" im Meter-Kilopond-Sekunden-System
-- (z.B. mit einer Geschwindigkeit von 4.0 Metern pro Sekunde, oder
-- mit einem Druck von 2.0 Kilopond pro Meter hoch 2, etc.) und
-- gibt einige dieser Zahlen zur aktuellen Ausgabe aus. Die letzte Ausgabe
-- loest eine Ausnahme DIMENSIONS_FEHLER aus.
---------------------------------------------------------------------------
   DIMENSIONS_FEHLER : exception;
   type HOCH_ZAHL is          range     -20 .. +20;
   type GLEIT     is digits 5 range -1.0e50 .. +1.0e50;
   ------------------------------------------------------------------------
   -- Ein uneingeschraenkter Verbunduntertyp mit 3 Diskriminanten:
   type ZAHL_MIT_EINHEIT(M, KP, SEC : HOCH_ZAHL := 0) is record
      ZAHL : GLEIT := 0.0;
   end record;
   ------------------------------------------------------------------------
   subtype DIMENSIONSLOS   is ZAHL_MIT_EINHEIT (M=> 0, KP=> 0, SEC=> 0);
   subtype LAENGE          is ZAHL_MIT_EINHEIT (M=> 1, KP=> 0, SEC=> 0);
   subtype FLAECHE         is ZAHL_MIT_EINHEIT (M=> 2, KP=> 0, SEC=> 0);
   subtype ZEIT            is ZAHL_MIT_EINHEIT (M=> 0, KP=> 0, SEC=> 1);
   subtype GESCHWINDIGKEIT is ZAHL_MIT_EINHEIT (M=> 1, KP=> 0, SEC=>-1);
   subtype BESCHLEUNIGUNG  is ZAHL_MIT_EINHEIT (M=> 1, KP=> 0, SEC=>-2);
   subtype KRAFT           is ZAHL_MIT_EINHEIT (M=> 0, KP=> 1, SEC=> 0);
   subtype DRUCK           is ZAHL_MIT_EINHEIT (M=>-2, KP=> 1, SEC=> 0);
   ------------------------------------------------------------------------
   DIML1            : DIMENSIONSLOS;    -- immer dim.los, nicht aenderbar
   LAENGE1          : LAENGE;           -- immer Laenge,  nicht aenderbar
   FLAECHE1         : FLAECHE;          -- immer Flaeche, nicht aenderbar
   ZEIT1            : ZEIT;             -- immer Zeit,    nicht aenderbar
   GESCHWINDIGKEIT1 : GESCHWINDIGKEIT;  -- immer Geschw., nicht aenderbar
   KRAFT1           : KRAFT;            -- immer Kraft,   nicht aenderbar
   DRUCK1           : DRUCK;            -- immer Druck,   nicht aenderbar
   FLEX1            : ZAHL_MIT_EINHEIT; -- anfangs dim.los, aenderbar!
   ------------------------------------------------------------------------
   function "*"(L, R: ZAHL_MIT_EINHEIT) return ZAHL_MIT_EINHEIT is
   begin
      return (M => L.M + R.M, KP => L.KP + R.KP, SEC => L.SEC + R.SEC,
              ZAHL => L.ZAHL * R.ZAHL);
   end "*";
   ------------------------------------------------------------------------
   function "/"(L, R: ZAHL_MIT_EINHEIT) return ZAHL_MIT_EINHEIT is
   begin
      return (M => L.M - R.M, KP => L.KP - R.KP, SEC => L.SEC - R.SEC,
              ZAHL => L.ZAHL / R.ZAHL);
   end "/";
   ------------------------------------------------------------------------
   function "+"(L, R: ZAHL_MIT_EINHEIT) return ZAHL_MIT_EINHEIT is
   begin
      if L.M = R.M and L.KP = R.KP and L.SEC = R.SEC then
         return (M => L.M, KP => L.KP, SEC => L.SEC,
                 ZAHL => L.ZAHL + R.ZAHL);
      else
         raise DIMENSIONS_FEHLER;
      end if;
   end "+";
   ------------------------------------------------------------------------
   procedure PUT_LINE(ITEM : ZAHL_MIT_EINHEIT) is
   -- Gibt ITEM in lesbarer Form zur aktuellen Ausgabe aus und beginnt
   -- dann eine neue Zeile (mit ada.text_io.new_line).
      package GLEIT_EA is new ada.text_io.float_io(num => GLEIT);
   begin  -- PUT
      GLEIT_EA.put(item => ITEM.ZAHL, exp => 0, fore => 3);
      ada.text_io.put ("  M hoch"   & HOCH_ZAHL'image(ITEM.M)   & " ");
      ada.text_io.put ("  KP hoch"  & HOCH_ZAHL'image(ITEM.KP)  & " ");
      ada.text_io.put ("  SEC hoch" & HOCH_ZAHL'image(ITEM.SEC) & " ");
      ada.text_io.new_line;
   end PUT_LINE;
   ------------------------------------------------------------------------
begin -- VERBU_02
   LAENGE1.ZAHL       := 12.0;                -- 12.0 Meter
   ZEIT1.ZAHL         :=  3.0;                --  3.0 Sekunden
   GESCHWINDIGKEIT1   := LAENGE1 / ZEIT1;     --  4.0 Meter pro Sekunde
   ------------------------------------------------------------------------
   KRAFT1.ZAHL        := 288.0;               -- 288.0 Kilopond
   FLAECHE1           := LAENGE1 * LAENGE1;   -- 144.0 Quadratmeter
   DRUCK1             := KRAFT1  / FLAECHE1;  --   2.0 KP pro M hoch 2
   ------------------------------------------------------------------------
   ada.text_io.put("LAENGE1          : "); PUT_LINE(LAENGE1);
   ada.text_io.put("ZEIT1            : "); PUT_LINE(ZEIT1);
   ada.text_io.put("GESCHWINDIGKEIT1 : "); PUT_LINE(GESCHWINDIGKEIT1);
   ada.text_io.put("KRAFT1           : "); PUT_LINE(KRAFT1);
   ada.text_io.put("FLAECHE1         : "); PUT_LINE(FLAECHE1);
   ada.text_io.put("DRUCK1           : "); PUT_LINE(DRUCK1);
   ------------------------------------------------------------------------
   FLEX1 := GESCHWINDIGKEIT1 * ZEIT1;
   ada.text_io.put(item => "LAENGE1          : "); PUT_LINE(LAENGE1);
   FLEX1 := DRUCK1 * (ZAHL => 144.0, M => 2, KP => 0, SEC => 0);
   ada.text_io.put(item => "KRAFT1           : "); PUT_LINE(KRAFT1);
   ------------------------------------------------------------------------
   FLEX1 := LAENGE1 + ZEIT1; -- Loest Ausnahme DIMENSIONS_FEHLER aus!
exception
   when DIMENSIONS_FEHLER =>
      ada.text_io.put_line("Die Ausnahme DIMENSIONS_FEHLER trat auf!");
end VERBU_02;