with ada.text_io;
procedure ZEIGR_08 is
---------------------------------------------------------------------------
-- Ein binaerer, sortierter Baum mit 15 Knoten wird aufgebaut und in Text-
-- form zur aktuellen Ausgabe ausgegeben. Dann wird 6 Mal im Baum gesucht
-- und die Suchergebnisse werden (ebenfalls in Textform) ausgegeben.
-- Der Baum besteht aus KNOTEN-Verbunden und ist nach der K1-Komponente
-- dieser Verbunde sortiert.
-- Ein leerer Baum besteht aus einem Abschlussknoten Z_LETZT.all. Dieser
-- Knoten wird nie geloescht. Alle "leeren" Zeiger im Baum zeigen auf
-- diesen Abschlussknoten. Vor jedem Suchen wird der gesuchte Wert in
-- den Abschlussknoten geschrieben.
---------------------------------------------------------------------------
   type DATENTYP is range 0..100;        -- Irgendein Datentyp.
   type KNOTEN;                          -- Unvollstaendige Typvereinbarung
   type ZAK      is access KNOTEN;       -- Zeiger auf Knoten
   type KNOTEN   is record               -- Vollstaendige Typvereinbarung
      K1:  DATENTYP;                     -- Datenkomponente 1
      K2:  DATENTYP;                     -- Datenkomponente 2
      LUB: ZAK;                          -- Zeiger auf linken  Unterbaum
      RUB: ZAK;                          -- Zeiger auf rechten Unterbaum
   end record;
   Z_LETZT: constant ZAK := new KNOTEN;  -- Zeiger auf den Abschlussknoten
   Z_ERST :          ZAK := Z_LETZT;     -- Zeiger auf den ersten  Knoten
   ------------------------------------------------------------------------
   package DATENTYP_EA is new ada.text_io.integer_io(num => DATENTYP);
   ------------------------------------------------------------------------
   procedure PUT(ITEM: KNOTEN) is
   -- Gibt ITEM in Textform zur akutellen Ausgabe aus.
   begin
      ada.text_io.put(item => "K1:");
      DATENTYP_EA.put(item => ITEM.K1);
      ada.text_io.put(item => "  K2:");
      DATENTYP_EA.put(item => ITEM.K2);
   end PUT;
   ------------------------------------------------------------------------   
   procedure PUT(ITEM: KNOTEN; WEG: string) is
   -- Gibt ITEM und WEG in Textform zur akutellen Ausgabe aus.
   begin
      PUT(ITEM => ITEM);
      ada.text_io.put(item => "  Weg: " & WEG);
      ada.text_io.new_line;
   end PUT;
   ------------------------------------------------------------------------
   procedure PUT(HIER: ZAK; WEG: string) is
   -- Gibt den binaeren Baum, auf dessen Wurzelknoten der Zeiger HIER
   -- zeigt, in Textform zur aktuellen Ausgabe aus.
   begin
      if HIER = Z_LETZT then -- Am "Ende des Baumes" angekommen!
         return;             -- Nichts mehr ausgeben.
      else
         -- Zuerst den linken Unterbaum ausgeben
         PUT(HIER => HIER.LUB, WEG => WEG & 'L');
         -- Dann den Wurzelknoten ausgeben:
         PUT(ITEM => HIER.all, WEG => WEG);
         -- Zuletzt den rechten Unterbaum ausgeben:
         PUT(HIER => HIER.RUB, WEG => WEG & 'R');
      end if;
   end PUT;
   ------------------------------------------------------------------------
   procedure FUEGE_EIN(HIER: in out ZAK; N1, N2: DATENTYP) is
   -- Fuegt einen neuen Knoten mit K1 gleich N1 und K2 gleich N2 in den
   -- binaeren Baum ein, auf dessen Wurzelknoten der Zeiger HIER zeigt.
   begin
      if HIER = Z_LETZT then             -- Am "Ende des Baums" angekommen!
         HIER := new KNOTEN'(K1 => N1, K2 => N2, LUB | RUB => Z_LETZT);
      elsif N1 < HIER.all.K1 then        -- Weiter nach links
         FUEGE_EIN(HIER => HIER.LUB, N1 => N1, N2 => N2);
      else                               -- Weiter nach rechts
         FUEGE_EIN(HIER => HIER.RUB, N1 => N1, N2 => N2);
      end if;
   end FUEGE_EIN;
   ------------------------------------------------------------------------
   procedure GEFUNDEN(HIER: ZAK) is
   -- Wird von der Prozedur SUCHE aufgerufen, wenn die K1-Komponente des
   -- Knotens HIER.all den gesuchten Wert hat:
   begin
      if HIER = Z_LETZT then             -- Nur am "Ende des Baums" im
         ada.text_io.put(item => "K1:"); -- Abschlussknoten gefunden!
         DATENTYP_EA.put(item =>  HIER.all.K1);
         ada.text_io.put(item => "           Knoten nicht gefunden!");
         ada.text_io.new_line;
      else                               -- Schluessel in einem "richtigen"
         PUT(ITEM => HIER.all);          -- Knoten gefunden!
         ada.text_io.put(item => "  Knoten gefunden!");
         ada.text_io.new_line;
      end if;
   end GEFUNDEN;
   ------------------------------------------------------------------------
   procedure SUCHE(HIER: ZAK; N1: DATENTYP) is
   -- Sucht in dem Baum, auf dessen Wurzel HIER zeigt, nach einem Knoten
   -- dessen K1-Komponente gleich N1 ist. Verlaesst sich darauf, dass es
   -- einen solchen Knoten gibt!
   begin
      if HIER.all.K1 = N1 then           -- N1 als K1-Komponente gefunden!
         GEFUNDEN(HIER => HIER);
      elsif N1 < HIER.all.K1 then        -- Weiter nach links
         SUCHE(HIER => HIER.LUB, N1 => N1);
      else                               -- Weiter nach rechts
         SUCHE(HIER => HIER.RUB, N1 => N1);
      end if;
   end SUCHE;
   ------------------------------------------------------------------------
   -- Daten zum Aufbauen des binaeren Baums und zum darin Suchen:
   type  DATEN_TAB is array(natural range <>) of DATENTYP;
   -- Aus den Daten in TAB1 und TAB2 wird der binaere Baum aufgebaut.
   -- Die Schluessel in TAB3 werden im Baum gesucht.
   TAB1: DATEN_TAB(1..15) :=  -- Die K1-Komponenten der Baumknoten
      (47, 25, 76, 14, 31, 84, 11, 28, 42, 80, 93, 13, 78, 82, 85);
   TAB2: DATEN_TAB(1..15) :=  -- Die K2-Komponenten der Baumknoten
      (01, 02, 03, 04, 05, 06, 07, 08, 09, 10, 11, 12, 13, 14, 15);
   TAB3: DATEN_TAB(1..6)  :=  -- K1-Komponenten zum Suchen
      (47, 13, 41, 84, 86, 93);
   ------------------------------------------------------------------------
begin
   ------------------------------------------------------------------------
   -- Den binaeren Baum aufbauen und ausgeben:
   for I in natural range TAB1'range loop
      FUEGE_EIN(HIER => Z_ERST, N1 => TAB1(I), N2 => TAB2(I));
   end loop;
   ada.text_io.put_line(item => "ZEIGR_08: Ein binaerer Baum: ----------");
   PUT(HIER => Z_ERST, WEG => "*");
   ------------------------------------------------------------------------
   -- Im binaeren Baum nach bestimmten Schluesseln (K1-Komponenten) suchen:
   ada.text_io.put_line(item => "ZEIGR_08: Suchergebnisse: -------------");
   for I in natural range TAB3'range loop
      Z_LETZT.all.K1 := TAB3(I);
      SUCHE(HIER => Z_ERST, N1 => TAB3(I));
   end loop;
   ada.text_io.put_line(item => "ZEIGR_08: Das war's erstmal! ----------");
end ZEIGR_08;